#    d2vbdiag.tcl  -  D2VB module diagnostic frame


set frame d2vbdiag
set frametitle "D2VB Diagnostics Frame"
set icontitle "D2VBdiag"

set mname "D2VB"
set DataWidth 32

if {[info exists env(D2VBserver)]} then {set resource $env(D2VBserver)} else {set resource vme}
if {[info exists env(RTOS)]} then {set RTOS $env(RTOS)} else {set RTOS LynxOS}

#    load the common f2vb/d2vb source

loadsource x2vbdiag

if {$RTOS == "VxWorks"} then {
    set X2VB_registerbase 0xf0e00000
    set X2VB_listrambase  0xf0e00800
} else {
    set X2VB_registerbase 0x00e00000
    set X2VB_listrambase  0x00e00800
}


proc define-x2vb-registers {} {
   global RTOS X2VB_registerbase X2VB_listrambase
   global resource mname dumpwords

   set register /$resource/$mname.D*
   set-footer "deleting register $register"
   EG "register :pattern '$register' !delete"

   define-register xVME DCSR "-a $X2VB_registerbase -o 0  -w 24 -d 16 -z x -p rw"
   define-register xVME DCCR "-a $X2VB_registerbase -o 2  -w 24 -d 16 -z x -p rw"
   define-register xVME DICR "-a $X2VB_registerbase -o 6  -w 24 -d 16 -z x -p rw"
   define-register xVME DIVR "-a $X2VB_registerbase -o 8  -w 24 -d 16 -z x -p rw"
   define-register xVME DLRAR "-a $X2VB_registerbase -o 10  -w 24 -d 16 -z x -p ro"
   define-register xVME DDRAR "-a $X2VB_registerbase -o 12  -w 24 -d 16 -z x -p rw"
   define-register xVME DCBRA "-a $X2VB_registerbase -o 16  -w 24 -d 32 -z x -p ro"
   define-register xVME DCBWA "-a $X2VB_registerbase -o 20  -w 24 -d 32 -z x -p ro"

   define-register bVME Dbram "-a [expr [read-register DDRAR] << 16] -w 32 -d 32 -z x -p rw -l $dumpwords"

if {$RTOS == "VxWorks"} then {
   define-register bVME Dlram "-a $X2VB_listrambase -w 16 -z x -p rw -l 96"
} else {
   define-register bVME Dlram "-a $X2VB_listrambase -w 24 -d 16 -z x -p rw -l 96"
}
}

proc define-register {class name attributes} {
   global resource mname

   set register /$resource/$mname.$name
   set-footer "defining register $register"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}


#    procedure taken from Euroball EC task

proc format_the_dump {} {
  global dumpdata
  global dumpopt dumpstart dumpwords

#    format dumpdata held in global variable dumpdata

   set-footer "formatting dump"

   set frame "dump"
   set win [self frame-name].$frame
   :frame $win :label  "Formatted Data Dump"  :transient T
   :layout T

   switch $dumpopt {
   0  {
        :galley G :rows 50 :cols 50 :deaf T :font F :strings ""

        :modify G :strings "raw dump starting at word offset 0\n"
        :modify G :strings $dumpdata
      }

   1  {
        :galley G :rows 50 :cols 50 :deaf T :strings ""
        set end [expr $dumpstart + $dumpwords]
        :modify G :strings "dump starting at word offset $dumpstart for $dumpwords words\n"
        set dump ""
        for {set i 0} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]
           set group [expr $word >> 16 & 255]
           set item  [expr $word >> 24 & 63]
           if {$group >= 248 && $item == 63} then {
#    last end token for event
             append dump "$word "
             :modify G :strings $dump
             :modify G :strings "\n\n"
             set dump ""
           } else {
             append dump "$word "
           }
        }
        :modify G :strings $dump
     }
   2  {
        :galley G :rows 50 :cols 60 :deaf T :strings ""
        set end [expr $dumpstart + $dumpwords]
        :modify G :strings "dump starting at word offset $dumpstart for $dumpwords words\n"
        for {set i 0} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]
           set group [expr $word >> 16 & 255]
           set item  [expr $word >> 24 & 63]
           set qbits [expr $word >> 30 & 3]
           set data  [expr $word & 0x0000ffff]
           set title "Unknown"
           if {$group <= 0} then {
#              invalid
           } elseif {$group <= 20} then {
#               Euroball III cards (Cluster)
              set detector [expr $group - 1]
              switch $item {
              0   {set title "Cluster$detector.GeA.20MeV"}
              1   {set title "Cluster$detector.GeA.4MeV"}
              2   {set title "Cluster$detector.GeA.FT"}
              3   {set title "Cluster$detector.GeB.20MeV"}
              4   {set title "Cluster$detector.GeB.4MeV"}
              5   {set title "Cluster$detector.GeB.FT"}
              6   {set title "Cluster$detector.GeC.20MeV"}
              7   {set title "Cluster$detector.GeC.4MeV"}
              8   {set title "Cluster$detector.GeC.FT"}
              9   {set title "Cluster$detector.GeD.20MeV"}
              10  {set title "Cluster$detector.GeD.4MeV"}
              11  {set title "Cluster$detector.GeD.FT"}
              12  {set title "Cluster$detector.GeE.20MeV"}
              13  {set title "Cluster$detector.GeE.4MeV"}
              14  {set title "Cluster$detector.GeE.FT"}
              15  {set title "Cluster$detector.GeF.20MeV"}
              16  {set title "Cluster$detector.GeF.4MeV"}
              17  {set title "Cluster$detector.GeF.FT"}
              18  {set title "Cluster$detector.GeG.20MeV"}
              19  {set title "Cluster$detector.GeG.4MeV"}
              20  {set title "Cluster$detector.GeG.FT"}
              21  {set title "Cluster$detector.BGOSum.Energy"}
              22  {set title "Cluster$detector.BGOSum.TAC"}
              23  {set title "Cluster$detector.BGO.Pattern.High"}
              24  {set title "Cluster$detector.BGO.Pattern.Low"}
              25  {set title "Cluster$detector.BGOA.Energy"}
              26  {set title "Cluster$detector.BGOA.TAC"}
              27  {set title "Cluster$detector.BGOB.Energy"}
              28  {set title "Cluster$detector.BGOB.TAC"}
              29  {set title "Cluster$detector.BGOC.Energy"}
              30  {set title "Cluster$detector.BGOC.TAC"}
              31  {set title "Cluster$detector.BGOD.Energy"}
              32  {set title "Cluster$detector.BGOD.TAC"}
              33  {set title "Cluster$detector.BGOE.Energy"}
              34  {set title "Cluster$detector.BGOE.TAC"}
              35  {set title "Cluster$detector.BGOF.Energy"}
              36  {set title "Cluster$detector.BGOF.TAC"}
              37  {set title "Cluster$detector.BGOG.Energy"}
              38  {set title "Cluster$detector.BGOG.TAC"}
              39  {set title "Cluster$detector.BGOH.Energy"}
              40  {set title "Cluster$detector.BGOH.TAC"}
              41  {set title "Cluster$detector.BGOI.Energy"}
              42  {set title "Cluster$detector.BGOI.TAC"}
              43  {set title "Cluster$detector.BGOJ.Energy"}
              44  {set title "Cluster$detector.BGOJ.TAC"}
              45  {set title "Cluster$detector.BGOK.Energy"}
              46  {set title "Cluster$detector.BGOK.TAC"}
              47  {set title "Cluster$detector.BGOL.Energy"}
              48  {set title "Cluster$detector.BGOL.TAC"}
              49  {set title "Cluster$detector.BGOM.Energy"}
              50  {set title "Cluster$detector.BGOM.TAC"}
              51  {set title "Cluster$detector.BGON.Energy"}
              52  {set title "Cluster$detector.BGON.TAC"}
              53  {set title "Cluster$detector.BGOO.Energy"}
              54  {set title "Cluster$detector.BGOO.TAC"}
              55  {set title "Cluster$detector.BGOP.Energy"}
              56  {set title "Cluster$detector.BGOP.TAC"}
              57  {set title "Cluster$detector.BGOQ.Energy"}
              58  {set title "Cluster$detector.BGOQ.TAC"}
              59  {set title "Cluster$detector.BGOR.Energy"}
              60  {set title "Cluster$detector.BGOR.TAC"}
              62  {set title "Cluster$detector.Ge.GIR.EvNum"}
              61  {set title "Cluster$detector.BGO.GIR.EvNum"}
              default  {}
              }
           } elseif {$group <= 50} then {
#               Eurogam II cards (Clover)
              set detector [expr $group - 21]
              switch $item {
              0   {}
              1   {}
              2   {}
              3   {}
              4   {set title "Clover$detector.BGO.Energy"}
              5   {set title "Clover$detector.BGO.TAC"}
              6   {set title "Clover$detector.BGO.Pattern"}
              7   {}
              8   {set title "Clover$detector.GeA.20MeV"}
              9   {set title "Clover$detector.GeA.4MeV"}
              10  {set title "Clover$detector.GeA.FT"}
              11  {set title "Clover$detector.GeA.CO"}
              12  {set title "Clover$detector.GeB.20Mev"}
              13  {set title "Clover$detector.GeB.4MeV"}
              14  {set title "Clover$detector.GeB.FT"}
              15  {set title "Clover$detector.GeB.CO"}
              16  {set title "Clover$detector.GeC.20MeV"}
              17  {set title "Clover$detector.GeC.4MeV"}
              18  {set title "Clover$detector.GeC.FT"}
              19  {set title "Clover$detector.GeC.CO"}
              20  {set title "Clover$detector.GeD.20MeV"}
              21  {set title "Clover$detector.GeD.4MeV"}
              22  {set title "Clover$detector.GeD.FT"}
              23  {set title "Clover$detector.GeD.CO"}
              default  {}
              }
           } elseif {$group <= 80} then {
#               Eurogam I cards  (Ge and BGO)
              set detector [expr $group - 51]
              switch $item {
              0   {set title "Ge$detector.20MeV"}
              1   {set title "Ge$detector.4MeV"}
              2   {set title "Ge$detector.FT"}
              3   {set title "Ge$detector.CO"}
              4   {set title "BGO$detector.Energy"}
              5   {set title "BGO$detector.TAC"}
              6   {set title "BGO$detector.Pattern"}
              default  {set title "unknown"}
              }  
           } elseif {$group <138} then {
#               Exogam Supression Shield cards
              set detector [expr $group - 128]
              switch $item {
              0   {set title "ESS$detector.Energy"}
              1   {set title "ESS$detector.Pattern"}
              2   {set title "ESS$detector.TDC.QA"}
              3   {set title "ESS$detector.TDC.QB"}
              4   {set title "ESS$detector.TDC.QC"}
              5   {set title "ESS$detector.TDC.QD"}
              6   {set title "ESS$detector.TDC"}
              62  {set title "ESS$detector.GIR.EvNum"}
              default  {set title "unknown"}
              }  
           } elseif {$group <170} then {
#               Exogam Centre Contact cards
              set detector [expr $group - 160]
              switch $item {
              0   {set title "ECC$detector.A.6MeV"}
              1   {set title "ECC$detector.A.20MeV"}
              2   {set title "ECC$detector.A.TAC"}
              3   {set title "ECC$detector.B.6MeV"}
              4   {set title "ECC$detector.B.20MeV"}
              5   {set title "ECC$detector.B.TAC"}
              6   {set title "ECC$detector.C.6MeV"}
              7   {set title "ECC$detector.C.20MeV"}
              8   {set title "ECC$detector.C.TAC"}
              9   {set title "ECC$detector.D.6MeV"}
              10  {set title "ECC$detector.D.20MeV"}
              11  {set title "ECC$detector.D.TAC"}
              62  {set title "ECC$detector.EVRI.EvNum"}
              default  {set title "unknown"}
              }  
           } elseif {$group <247} then {
#               ancillary detectors
              set detector [expr $group - 81]
              switch $item {
              0   {set title "Saphir$detector.Energy"}
              1   {set title "Saphir$detector.Timing"}   
              62  {set title "Saphir$detector.GIR.EvNum"}   
              default {set title "unknown"}  
              }    
           } elseif {$group == 247} then {
#               trigger
           } else {
#               tokens 
              if {$item <= 15} then {set title "start token"} \
              elseif {$item <= 31} then {set title "event number lsw"} \
              elseif {$item <= 47} then {set title "event number msw"} \
              else {set title "end token"}
           }
           :modify G :strings "[format "%10s %3d %3d %4d %6x %24s" $word $qbits $item $group $data $title] \n"
           if {$item == 63} then {:modify G :strings "\n"}
       }
     }

   }
   :modify G :see 0
   :layout B; :set fill none; :set anchor center
   :button B  :label "Dismiss" :command "destroy $win"
   :show T
   clear-footer
}

